REBOL [
	Name: "Uniserve Monitor"
	Author: "Nenad Rakocevic / SOFTINNOV"
	Copyrights: "SOFTINNOV"
	Version: 0.2
	Date: 27/09/2004
]

do %../uni-engine.r
do %../libs/win95.r

uniserve-path: %../
uniserve/boot/no-loop/with [protocols [admin]]

;--- Hosts list management ---

hosts: context [
	file: %hosts.dat
	list: load file
	add: func [host port][
		repend/only list [host port]
		save file list
	]
	del: func [pos][
		remove at list pos
		save file list
	]
]

;--- Client/server Communications ---

server: context [
	admin-port: host: connected?: none
	
	process: func [msg cmd][
		switch cmd [
			list  [uni-servers/build msg]
			start [server/send [list]]
			stop  [server/send [list]]
			stats []
		]
	]
	
	events: [
		on-logged: does [
			connected?: yes
			set-led-color green
			set-activity off
			set-info reform ["Connected to" host]
			send [list]
			main/set-tab 2
		]
		on-response: func [msg cmd][
			set-activity off
			process msg cmd
		]
		on-error: func [reason [word!]][
			connected?: no
			set-led-color red
			set-activity off
			set-info reform [
				"ERROR:" select [
					unreachable		"host is unreachable !"
					unknown-host	"cannot resolve host name !"
					connect-failed	"cannot connect to host !"
				] reason
			]
		]
		on-close-server: does [
			connected?: no
			set-activity off
			set-led-color red
			set-info "Connection closed."
			services/data: none
			services/refresh
		]
	]
		
	send: func [cmd][
		insert-port admin-port cmd
		set-activity on
	]

	connect: func [id][
		set-led-color yellow
		set-activity on
		set-info reform ["Connecting to" host: id/1 "..."]
		if port? admin-port [
			close-port admin-port
		]	
		admin-port: open-port join admin:// [host #":" id/2] events
	]
]

;--- Services Management ---

uni-servers: context [
	list: make block! 8
	
	build: func [blk /local name port mod run?][
		clear list	
		foreach [port name mod run?] blk [	
			repend/only list [
				name port any [mod ""] 
				any [all [run? = 'true 'yes] 'no]
			]
		]	
		services/data: list
		services/refresh
	]
	
	start: func [n][
		server/send reduce ['start list/:n/1 list/:n/2]
	]
	
	stop: func [n][
		server/send reduce ['stop list/:n/1 list/:n/2]
	]
	
	toggle: func [n][
		either list/:n/4 = 'yes [stop n][start n]
	]
]

;--- GUI Helper Functions ---

set-activity: func [mode [logic!]][
	activity/pane/set-mode mode
]

set-info: func [msg][
	info/text: msg
	show info
]

leds: reduce [
	'red	load %imgs/led-red-small.png 
	'yellow	load %imgs/led-yellow-small.png 
	'green 	load %imgs/led-green-small.png 
]

set-led-color: func ['color [word!]][
	change at led/effect/draw 2 leds/:color
	show led
]

;--- Extension of the stylesheet ---

win95: stylize/styles [

	url: txt font [color: winskin/colors/high styles: 'underline]

	sbox: box 18x18 edge [size: 1x1] with [
		init: [
			font: make templates/text/font []
			edge: make edge []
			edge/effect: winskin/ibevel-effect self
		]
	]
	
	ping-pong-anim: box 1x18 feel [
		redraw: detect: over: east?: none
		engage: func [f a e][		
			if a = 'time [		
				f/size/x: f/size/x + 2
				if east? [f/offset/x: f/offset/x - 2]
				if f/size/x = 53 [			
					east?: not east?
					change at f/effect 2 pick [-1x0 1x0] east?
					f/offset/x: pick [54 0] east? 
					f/size/x: 1
				]
				show f
			]
		]
	] with [
		set-mode: func [mode [logic!]][
			either mode [
				rate: 20
				effect: effects
			][
				rate: effect: feel/east?: none
				size/x: 1
				offset/x: 0
			]
			show self
		]
		offset: 0x0
		effects: reduce [
			'gradient 1x0
			winskin/colors/back
			winskin/colors/high
		]
	]
] win95

;--- Forms layouts ---

forms: context [
	service: port: module: svc: none

	add-service: [
		styles win95
		style txt text 80 right
		origin 10x10
		space 10x20
		backcolor winskin/colors/back
		pad 0x10
		across
		txt "Services:" service: droplist no-edit 120 "<Choose a Service>" [
			svc: pick uni-servers/list service/picked
			port/reset
			module/reset
			if svc [
				port/add-item svc/2 svc/2
				port/refresh
				show port
			]
		] return
		txt "Port:" port: droplist 60 return
		txt "Module:" module: droplist no-edit 120 "<Choose a Module>" return
		pad 0x20
		
		button "Create" [
			server/send reduce [
				'start
				first pick uni-servers/list service/picked
				to-integer port/text
			]
			hide-popup
		]
		button "Cancel" [hide-popup]
		do [
			svc: uni-servers/list
			forall svc [service/add-item svc/1/1 index? svc]
			service/refresh
		]
	]
	
	show-me: func ['name][
		show-popup center-face layout get bind name 'self
		do-events
	]
]

;--- Main layout ---

view/title center-face layout [
	styles win95
	;style frz-but button 80x17 feel [engage: none]
	size 376x301
	origin 0x10
	backcolor winskin/colors/back
	below
	main: tab-panel 378x270 [
		" Servers " [
			origin 10x10
			tbl-hosts: table [
				["Host" 200 left]["Port" 60]
			] data hosts/list 
			return
			button "Connect..." 60 [
				if tbl-hosts/picked [server/connect tbl-hosts/picked]
			]
			button "Add" 60
			button "Remove" 60
			button "Properties" 60
		]
		" Control " [
			origin 10x10
			services: table [
				["Services" 80 left]
				["Port" 40 left]
				["Module" 80 left]
				["Running?" 60 center]
			] return
			sbb: box red pad 0x-106
			sb: button "Start/Stop" 60 [
				either services/selected [
					uni-servers/toggle services/selected
				][
					set-info "Warning: No service selected !"
				]
			]
			button "Add" 60 [
				either server/connected? [
					forms/show-me add-service
				][
					set-info "Warning: Not connected !"
				]
			]
			button "Properties" 60
			button "Logs" 60
			
			do [
				sbb/size: sb/size + 2x2
				sbb/offset: sb/offset - 1x1
			]
		]
		;" Statistics " []
		" About " [
			pad 60x20
			txt font [size: 20 styles: 'bold] 
				join "UniServe Monitor v" system/script/header/version
			txt "Author: Nenad Rakocevic / SOFTINNOV"
			url "More information here" [browse http://softinnov.org]
		]
	] 
	pad 0x-7
	status-pan: panel 376x20 [
		styles win95
		origin 0x0
		space 0x0
		across
		info: sbox 300x18 "Not connected!"
		activity: sbox 54x18 with [
			pane: make-face/styles 'ping-pong-anim win95
		]
		led: sbox 20x18 effect [draw [image leds/red 3x2]]
	]	
	
;	do [forms/show-me add-service]
	
] reform ["Uniserve Monitor v" system/script/header/version]