REBOL [
    Title: "Log Object Definition"
    Purpose: "Provides a easy way to log messages to screen/file"
    File:  %log.r
    Version: 1.0.0
]


logger: context [
	level: 'screen	; possible are : 'screen, 'file, 'csv
	file: %activity
	file.log: join copy file %.log
	file.csv: join copy file %.csv
	
	notify: func [msg module [word!] type [word!] /local txt out][
		if block! = type? msg: reduce msg [msg: rejoin msg]
		out: join uppercase to-string msg/1 next msg
		switch type [
			warn  [txt: ["# Warning in " mold reduce [module] " : " out " !"]]
			error [txt: ["## Error in " mold reduce [module] " : " out " !"]]
			info  [txt: [mold reduce [module] " " out]]
		]
		switch level [	
			screen [
				system/words/print rejoin txt
			]		
			file [
				write/append file.log rejoin [now " " rejoin txt #"^/"]
			]		
			both [
				system/words/print rejoin txt
				write/append file.log rejoin [now " " rejoin txt #"^/"]
			]
			csv [
				switch type [
					warn	[type: 'Warning]
					error	[type: 'Error]
					info	[type: 'Info]
				]
				write/append file.csv rejoin [
					mold now ";"
					mold type ";"
					mold module ";"
					mold join uppercase msg/1 lowercase next msg
				]
			]
		]
	]
]

log-class: context [
	name: 'log-class
	
	log: func [msg /warn /error /info][
		if warn  [logger/notify msg name 'warn]
		if error [logger/notify msg name 'error]
		if info  [logger/notify msg name 'info]
		exit
	]
	
]