REBOL [
	Title: "Uniserve: HTTP proxy service"
	Author: "SOFTINNOV / Nenad Rakocevic"
	Version: 1.0.0
	Date: 27/08/2004
]

install-service [
	name: 'Proxy-HTTP
	port-id: 8080
	
	blank: #" "
	out-buffer: make string! 100 * 1024
	
	msg-no-host: "HTTP/1.0 200 OK^M^/^M^/<html><h1>Proxy error: Host unreachable !</h1></htlm>"
	
	proto-ctx: context [
		method: 'GET
		headers: posted: url: request: auth-token: status-line: none
	]
	
	reset-stop: does [stop-at: newline]
	
	http-events: [
		on-send: func [port request /local client name value][
			client: port/client-port
			foreach [name value] port/ctx/headers [
				if not find [Proxy-Connection] name [
					h-store request/headers name value
				]
			]
			h-store request/headers 'Connection "close"
			port/user-data/out/method: client/user-data/ctx/method
			port/user-data/out/content: client/user-data/ctx/posted
		]
		on-header-end: func [port res /local client pos][
			client: port/client-port
			if pos: find res/status-line "1.1" [
				change/part pos "1.0" 3
			]
			h-store res/headers 'Proxy-Connection "close"
			write-peer/with 
				rejoin [res/status-line crlf form-header res/headers]
				client
		]
		on-stream-in: func [port data /local client][
			write-peer/with copy data port/client-port
		]
		on-response: func [port res /local client][
			client: port/client-port
			if res/content [
				write-peer/with res/content port/client-port
			]
			close-peer/with port/client-port
		]
		on-error: func [port reason /local client][
			client: port/client-port
			;?? reason
			write-peer/with msg-no-host client
			close-peer/with client
		]
	]	
	
	connect-server: has [session valid method url][
		session: client/user-data
		if not valid: parse/all session/ctx/status-line [
			copy method to blank skip copy url ["http://" to blank] to end
		][
			; invalid url error handling
			;make error! join "invalid url: " session/ctx/status-line
			close-client
			return
		]
		session/server-port: open-port/with url http-events [
			client-port: client
			ctx: session/ctx
		]
	]
	
	on-new-client: does [
		reset-stop
		client/user-data: context [
			server-port: none
			ctx: make proto-ctx [
				headers: make block! 8
			]
		]
	]
	
;	on-close-client: has [cu][
;		if all [
;			cu: client/user-data
;			cu/server-port
;			cu/server-port/client-port
;		][
;			cu/server-port/client-port: none
;			close-peer/force/with cu/server-port
;		]
;	]
	
	on-received: func [data /locals session len name value pos][
		either integer? stop-at [
			client/user-data/ctx/posted: copy data
			connect-server
			reset-stop
		][
			session: client/user-data
			trim/with data #"^/"
			trim/with data #"^M"

			either empty? data [
				either len: select session/ctx/headers 'Content-Length [
					stop-at: to integer! len
				][
					connect-server
				]
			][
				either session/ctx/status-line [
					name: to word! to string! copy/part data pos: find data #":"
					value: trim to string! copy next pos
					h-store session/ctx/headers name value
				][ 
					session/ctx/status-line: to string! data
					session/ctx/method: to word! to string! copy/part data find data blank
				]
			]
		]
	]
	
]
