REBOL [
	Title: "Uniserve: task-master client"
	Author: "SOFTINNOV / Nenad Rakocevic"
	Version: 1.0.0
	Date: 27/12/2002
]

system/options/quiet: true

change-dir %../..	; go back to uniserve root directory
do %libs/log.r

s-read-io: get in system/words 'read-io

ctx-task-class: make log-class [
	name: 'task-handler
	verbose: 0
	
	module: server: scheme: state: err: req: t0: len: remains: none
	packet: make binary! buf-size: 1024 * 16
	request: make binary! buf-size
	
	server-address: 127.0.0.1
	server-port: 9799
	
	modules: make block! 5
	
	set 'install-module func [body [block!]][
		module: make make log-class [on-task-received: response: none] body
		if verbose > 0 [log/info ["installing module : " mold module/name]]
		repend modules [module/name module]
		recycle
	]

	send-data: func [data][
		insert server debase/base to-hex length? data 16
		insert server data
	]

	connect: does [
		if error? try [
			server: open/binary/direct/no-wait rejoin [tcp:// server-address ":" server-port]
		][quit]
		log/info "connected to Uniserve"
		forever [
			clear request
			clear packet
			if verbose > 0 [log/info "waiting for task..."]
			wait server
			state: s-read-io server packet 4
			any [
				all [
					state <= 0
					close server
					quit
				]
				all [
					state = 4
					remains: len: to integer! packet
				]
				all [
					state < 4
					len: to integer! copy/part packet 4
					remains: len - length? packet: at packet 5
					insert tail request packet
				]
			]
			if verbose > 0 [log/info ["head received, body:" len]]
			until [
				clear packet
				wait server
				state: s-read-io server packet min remains buf-size
				if verbose > 1 [log/info ["state:" state]]
				if positive? state [
					remains: remains - length? packet
					insert tail request packet
				]
				any [zero? remains state <= 0]
			]
			;if negative? state [close server quit]
			if verbose > 0 [t0: now/time/precise]
			req: load to string! request
			either error? err: try [
				if not find modules req/2 [do join %modules/ [req/2 %.r]]
				module: select modules req/2
				module/response: none
				module/on-task-received load to string! req/3
				false
			][
				err: disarm err
				log/error mold err
				send-data mold reduce ['error err]
			][
				send-data mold reduce ['ok module/response]
				if verbose > 0 [log/info ["done in " now/time/precise - t0 " sec"]]
			]
			recycle
		]
	]
]

ctx-task-class/connect
